package com.jiuqi.util.patterns;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.jiuqi.util.patterns.ConstPattern;
import com.jiuqi.util.patterns.IPattern;
import com.jiuqi.util.patterns.ITranslator;
import com.jiuqi.util.patterns.PatternException;
import com.jiuqi.util.patterns.VariablePattern;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

class PlaceholderPattern implements IPattern {
  private List items = new ArrayList();
  
  private String expression;
  
  private boolean isVariable;
  
  public String toString() {
    StringBuffer buffer = new StringBuffer();
    if (this.isVariable)
      buffer.append("${"); 
    for (int i = 0; i < size(); i++)
      buffer.append(get(i).toString()); 
    if (this.isVariable)
      buffer.append("}"); 
    return buffer.toString();
  }
  
  public boolean isVariable() {
    return this.isVariable;
  }
  
  public void setIsVariable(boolean value) {
    this.isVariable = value;
  }
  
  public void parse(String expression) throws PatternException {
    clear();
    this.expression = expression;
    if (expression == null)
      return; 
    int cur = 0;
    int start = 0;
    int stackSize = 0;
    boolean startVar = false;
    while (cur < expression.length()) {
      switch (charOf(cur)) {
        case '$':
          if (charOf(cur + 1) == '{') {
            if (startVar) {
              stackSize++;
            } else {
              addConst(start, cur);
              start = cur;
              startVar = true;
              stackSize = 0;
            } 
            cur += 2;
            continue;
          } 
          cur++;
          continue;
        case '}':
          if (startVar)
            if (stackSize == 0) {
              addVariable(start + 2, cur);
              startVar = false;
              start = cur + 1;
            } else {
              stackSize--;
            }  
          cur++;
          continue;
      } 
      cur++;
    } 
    if (cur == expression.length() && cur > start)
      addConst(start, cur); 
  }
  
  private char charOf(int index) {
    return (index < 0 || index >= this.expression.length()) ? Character.MIN_VALUE : this.expression.charAt(index);
  }
  
  private void addConst(int start, int cur) {
    if (start >= cur)
      return; 
    String s = this.expression.substring(start, cur);
    ConstPattern p = new ConstPattern(s);
    add((IPattern)p);
  }
  
  private void addVariable(int start, int cur) throws PatternException {
    if (start >= cur)
      return; 
    String s = this.expression.substring(start, cur);
    int p1 = s.indexOf("${");
    int p2 = s.lastIndexOf('}');
    if (p1 >= 0 && p1 < p2) {
      com.jiuqi.util.patterns.PlaceholderPattern p = new com.jiuqi.util.patterns.PlaceholderPattern();
      p.parse(s);
      p.setIsVariable(true);
      add(p);
    } else {
      VariablePattern p = new VariablePattern();
      p.parse(s);
      add((IPattern)p);
    } 
  }
  
  public void add(IPattern pattern) {
    this.items.add(pattern);
  }
  
  public void add(int index, IPattern pattern) {
    this.items.add(index, pattern);
  }
  
  public void clear() {
    this.items.clear();
    this.expression = null;
    this.isVariable = false;
  }
  
  public IPattern get(int index) {
    return this.items.get(index);
  }
  
  public String getExpression() {
    return toString();
  }
  
  public int getLength() {
    return 0;
  }
  
  public int getType() {
    return 1;
  }
  
  public int indexOf(IPattern pattern) {
    return this.items.indexOf(pattern);
  }
  
  public void remove(int index) {
    this.items.remove(index);
  }
  
  public void setExpression(String value) {
    try {
      parse(value);
    } catch (PatternException patternException) {
      clear();
    } 
  }
  
  public void setLength(int value) {}
  
  public int size() {
    return this.items.size();
  }
  
  public int match(String s, int start, Map<String, String> values) {
    if (this.isVariable)
      return -1; 
    int cur = 0;
    int next = start;
    while (cur < size()) {
      int backLen;
      String value;
      switch (get(cur).getType()) {
        case 0:
          next = get(cur).match(s, start, values);
          if (next == -1)
            return -1; 
          break;
        case 1:
          if (get(cur).getLength() > 0) {
            next = get(cur).match(s, start, values);
            if (next == -1)
              return -1; 
            String str = s.substring(start, next);
            values.put(get(cur).getExpression(), str);
            break;
          } 
          if (cur >= size() - 1) {
            String str = s.substring(start);
            values.put(get(cur).getExpression(), str);
            next = s.length();
            break;
          } 
          if (get(cur + 1).getType() == 0) {
            next = s.indexOf(get(cur + 1).getExpression(), start + 1);
            if (next == -1)
              return -1; 
            String str = s.substring(start, next);
            values.put(get(cur).getExpression(), str);
            break;
          } 
          backLen = statNextLength(cur);
          if (backLen == -1 || start >= s.length() - backLen)
            return -1; 
          next = s.length() - backLen;
          value = s.substring(start, next);
          values.put(get(cur).getExpression(), value);
          break;
      } 
      start = next;
      cur++;
    } 
    return start;
  }
  
  private int statNextLength(int cur) {
    int len = 0;
    for (int i = cur + 1; i < size(); i++) {
      switch (get(i).getType()) {
        case 0:
          len += get(i).getLength();
          break;
        case 1:
          if (get(i).getLength() > 0) {
            len += get(i).getLength();
            break;
          } 
          return -1;
        default:
          return -1;
      } 
    } 
    return len;
  }
  
  public String replace(ITranslator translator) {
    StringBuffer buffer = new StringBuffer();
    for (int i = 0; i < size(); i++)
      buffer.append(get(i).replace(translator)); 
    if (this.isVariable) {
      String[] ret = translator.translate(buffer.toString(), 0);
      if (ret == null)
        return buffer.toString(); 
      return (ret.length == 0) ? "" : ret[0];
    } 
    return buffer.toString();
  }
}
